import scrapy
from scrapy.crawler import CrawlerProcess
from playwright._impl._api_types import TimeoutError

import pkg_resources


class SpotifyLinksSpider(scrapy.Spider):

    name = "spotify_links"
    start_urls = [
        "https://www.lifeatspotify.com/jobs?c=engineering&c=data&l=remote-emea&l=stockholm&l=london&l=berlin&l=amsterdam&l=sweden-home-mix&l=copenhagen&l=germany-home-mix&l=barcelona&l=gothenburg"
    ]
    spotify_links = set()

    def start_requests(self):
        yield scrapy.Request(
            self.start_urls[0],
            self.parse,
            meta={"playwright": True, "playwright_include_page": True},
        )

    async def parse(self, response):
        page = response.meta["playwright_page"]
        print(page)

        while True:
            try:
                load_more_locator = page.locator(
                    '//button[@aria-label="Load more jobs"]'
                )
                await load_more_locator.click()
            except TimeoutError:
                print("No more jobs to load.")
                break

        links = await page.query_selector_all("//a")
        for link in links:
            href = await link.get_attribute("href")
            if href.startswith("/jobs/"):
                self.spotify_links.add("https://www.lifeatspotify.com" + href)

        stream = pkg_resources.resource_stream(__name__, 'data/spotify_links.txt')
        with open(stream.name, "w") as f:
            f.write(str(self.spotify_links))


if __name__ == "__main__":
    process = CrawlerProcess(
        settings={
            "TWISTED_REACTOR": "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
            "DOWNLOAD_HANDLERS": {
                "https": "scrapy_playwright.handler.ScrapyPlaywrightDownloadHandler",
            },
            "CONCURRENT_REQUESTS": 32,
            "ROBOTSTXT_OBEY": False,
            "AUTOTHROTTLE_ENABLED": True,
            "AUTOTHROTTLE_TARGET_CONCURRENCY": 1,
            "AUTOTHROTTLE_START_DELAY": 5,
            "AUTOTHROTTLE_MAX_DELAY": 60,
        }
    )
    process.crawl(SpotifyLinksSpider)
    process.start()



import psycopg2

from config.definitions import JOB_MARKET_DB_PWD, JOB_MARKET_DB_USER


class JobsCrawlerPipeline:

    def __init__(self):
        self.connection = None
        self.cur = None

    def open_spider(self, spider):
        hostname = 'localhost'
        username = JOB_MARKET_DB_USER
        password = JOB_MARKET_DB_PWD
        database = 'job_market'
        self.connection = psycopg2.connect(
            host=hostname, user=username, password=password,
            dbname=database)
        self.cur = self.connection.cursor()

    def close_spider(self, spider):
        self.cur.close()
        self.connection.close()

    def process_item(self, item, spider):
        for field in item.fields:
            item.setdefault(field, 'NULL')
        try:
            self.cur.execute(
                "INSERT INTO raw_jobs(url, title, company, location, type, industry, text, remote, created_at) "
                "VALUES(%s,%s,%s,%s,%s,%s,%s,%s,%s) "
                "ON CONFLICT (url) DO UPDATE "
                "SET title = EXCLUDED.title;",
                (item['url'][0], item['title'][0], item['company'][0], item['location'][0], item['type'][0],
                 item['industry'][0], item['text'][0], item['remote'][0], item['created_at'][0]))
            self.connection.commit()
        except:
            self.connection.rollback()
            raise
        return item



import scrapy
import ast
import pkg_resources
from datetime import datetime

from scrapy.crawler import CrawlerProcess
from scrapy.loader import ItemLoader
from itemloaders.processors import Join

from data_job_crawler.crawler.items import JobsCrawlerItem


class SpotifySpider(scrapy.Spider):

    name = "spotify"

    @staticmethod
    def extract_links():
        stream = pkg_resources.resource_stream(__name__, 'data/spotify_links.txt')
        with open(stream.name, "r") as f:
            links = ast.literal_eval(f.read())
            return links

    def start_requests(self):
        links = self.extract_links()
        for link in links:
            yield scrapy.Request(link, self.yield_job_item)

    def yield_job_item(self, response):
        l = ItemLoader(item=JobsCrawlerItem(), response=response)

        l.add_value("url", response.url)

        title = response.xpath("//h1/span/text()").get()
        subtitle = response.xpath("//h1/text()").get()
        if subtitle:
            job_title = title + "- " + subtitle
        else:
            job_title = title
        l.add_value("title", job_title)

        l.add_value("company", "Spotify")

        remote = response.xpath('//span[text()="Remote EMEA"]').get()
        if remote == "Remote EMEA":
            l.add_value("remote", "Remote EMEA")

        l.add_value(
            "location",
            response.xpath('//p[text()="Location"]/parent::div//span/text()').get(),
        )

        l.add_value(
            "type",
            response.xpath('//p[text()="Job type"]/following-sibling::p/text()').get(),
        )

        l.add_value("industry", "NA")

        l.add_value(
            "text",
            response.xpath(
                '//div[contains(@class, "singlejob_rightContent")]/div[1]//text()'
            ).getall(),
            Join(),
        )

        l.add_value("created_at", datetime.now())

        yield l.load_item()


if __name__ == "__main__":
    process = CrawlerProcess(
        settings={
            "ROBOTSTXT_OBEY": False,
            "ITEM_PIPELINES": {
                "data_job_crawler.crawler.pipelines.JobsCrawlerPipeline": 300,
            },
            "AUTOTHROTTLE_ENABLED": True,
            "AUTOTHROTTLE_TARGET_CONCURRENCY": 1,
            "AUTOTHROTTLE_START_DELAY": 5,
            "AUTOTHROTTLE_MAX_DELAY": 60,
        }
    )
    process.crawl(SpotifySpider)
    process.start()



import scrapy
import ast
import pkg_resources
from datetime import datetime

from scrapy.crawler import CrawlerProcess
from scrapy.loader import ItemLoader
from itemloaders.processors import Join

from data_job_crawler.crawler.items import JobsCrawlerItem


class WttjSpider(scrapy.Spider):
    """
    Spider to scrape jobs information on Welcome to the Jungle Website.
    The individual pages are not rendered with Javascript so it only uses Scrapy.
    """

    name = "wttj"

    @staticmethod
    def extract_links():
        stream = pkg_resources.resource_stream(__name__, 'data/wttj_links.txt')
        with open(stream.name, "r") as f:
            links = ast.literal_eval(f.read())
            return links

    def start_requests(self):
        links = self.extract_links()
        for link in links:
            yield scrapy.Request(link, self.yield_job_item)

    def yield_job_item(self, response):
        l = ItemLoader(item=JobsCrawlerItem(), response=response)
        l.add_value("url", response.url)
        l.add_value(
            "title",
            response.xpath(
                '//*[text()="Le poste"]/parent::h4/following-sibling::h4/text()'
            ).get(),
        )
        l.add_value(
            "company",
            response.xpath(
                '//*[@data-testid="job-summary-organization-title"]/text()'
            ).get(),
        )
        l.add_value(
            "remote",
            response.xpath(
                '//*[@name="remote"]/parent::span/following-sibling::span//text()'
            ).get(),
        )
        l.add_value(
            "location",
            response.xpath(
                '//*[@name="location"]/parent::span/following-sibling::span/text()'
            ).get(),
        )
        l.add_value(
            "type",
            response.xpath(
                '//*[@name="contract"]/parent::span/following-sibling::span/span/text()'
            ).get(),
        )
        l.add_value(
            "industry",
            response.xpath(
                '//*[@name="tag"]/parent::span/following-sibling::span/text()'
            ).get(),
        )
        l.add_value(
            "text",
            response.xpath("//h2/following-sibling::div//text()").getall(),
            Join(),
        )
        l.add_value("created_at", datetime.now())
        yield l.load_item()


if __name__ == "__main__":
    process = CrawlerProcess(
        settings={
            "ROBOTSTXT_OBEY": False,
            "ITEM_PIPELINES": {
                "data_job_crawler.crawler.pipelines.JobsCrawlerPipeline": 300,
            },
            "AUTOTHROTTLE_ENABLED": True,
            "AUTOTHROTTLE_TARGET_CONCURRENCY": 1,
            "AUTOTHROTTLE_START_DELAY": 5,
            "AUTOTHROTTLE_MAX_DELAY": 60,
        }
    )
    process.crawl(WttjSpider)
    process.start()



import scrapy
from scrapy.crawler import CrawlerProcess

import pkg_resources


class WttjLinksSpider(scrapy.Spider):
    """
    This Spider is used to render Javascript. It outputs all job links into a file.
    """

    name = "wttj_links"
    start_urls = [
        "https://www.welcometothejungle.com/fr/jobs?page={page_number}&aroundQuery=&query=data%20engineer&refinementList%5Bcontract_type_names.fr%5D%5B%5D=CDI&refinementList%5Bcontract_type_names.fr%5D%5B%5D=CDD%20%2F%20Temporaire&refinementList%5Bcontract_type_names.fr%5D%5B%5D=Autres&refinementList%5Bcontract_type_names.fr%5D%5B%5D=VIE&refinementList%5Bcontract_type_names.fr%5D%5B%5D=Freelance"
    ]

    BASE_URL = "https://www.welcometothejungle.com"

    links = set()

    def start_requests(self):
        yield scrapy.Request(
            self.start_urls[0].format(page_number=1),
            self.parse_jobs_list,
            meta={"playwright": True, "playwright_include_page": True},
        )

    async def parse_jobs_list(self, response):
        """Parse javascript rendered results page and obtain individual job page links."""
        page = response.meta["playwright_page"]

        job_elements = await page.query_selector_all(
            '//*[@class="ais-Hits-list-item"]//a'
        )
        for job_element in job_elements:
            job_link = await job_element.get_attribute("href")
            job_url = self.BASE_URL + job_link
            self.links.add(job_url)

        while True:
            try:
                next_locator = page.locator('//*[@aria-label="Next page"]')
                async with page.expect_navigation():
                    await next_locator.click()

                job_elements = await page.query_selector_all(
                    '//*[@class="ais-Hits-list-item"]//a'
                )
                for job_element in job_elements:
                    job_link = await job_element.get_attribute("href")
                    job_url = self.BASE_URL + job_link
                    self.links.add(job_url)
            except TimeoutError:
                print("Cannot find a next button on ", page.url)
                break
            finally:
                stream = pkg_resources.resource_stream(__name__, 'data/wttj_links.txt')
                with open(stream.name, "w") as f:
                    f.write(str(self.links))

        await page.close()


if __name__ == "__main__":
    process = CrawlerProcess(
        settings={
            "TWISTED_REACTOR": "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
            "DOWNLOAD_HANDLERS": {
                "https": "scrapy_playwright.handler.ScrapyPlaywrightDownloadHandler",
            },
            "CONCURRENT_REQUESTS": 32,
            "ROBOTSTXT_OBEY": False,
            "AUTOTHROTTLE_ENABLED": True,
            "AUTOTHROTTLE_TARGET_CONCURRENCY": 1,
            "AUTOTHROTTLE_START_DELAY": 5,
            "AUTOTHROTTLE_MAX_DELAY": 60,
        }
    )
    process.crawl(WttjLinksSpider)
    process.start()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['data_job_crawler',
 'data_job_crawler.crawler',
 'data_job_crawler.crawler.spiders']

package_data = \
{'': ['*'], 'data_job_crawler.crawler.spiders': ['data/*']}

install_requires = \
['Scrapy>=2.6.2,<3.0.0', 'playwright>=1.24.1,<2.0.0', 'psycopg2>=2.9.3,<3.0.0']

setup_kwargs = {
    'name': 'data-job-crawler',
    'version': '0.3.0',
    'description': '',
    'long_description': None,
    'author': 'Donor',
    'author_email': 'donorfelita@msn.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# Define here the models for your scraped items
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/items.html

import scrapy


class JobsCrawlerItem(scrapy.Item):
    url = scrapy.Field()
    title = scrapy.Field()
    company = scrapy.Field()
    location = scrapy.Field()
    type = scrapy.Field()
    industry = scrapy.Field()
    text = scrapy.Field()
    remote = scrapy.Field()
    created_at = scrapy.Field()


